library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity TFTCtl is 
	port (
          clk : IN std_logic;
          rst	: IN std_logic;
          DE_O : out std_logic;
          CLK_O : out std_logic;
          DISP_O : out std_logic;
          VDDEN_O : out std_logic;
          R_O : out std_logic_vector(7 downto 0);
          G_O : out std_logic_vector(7 downto 0);          
          B_O : out std_logic_vector(7 downto 0)

          );
	
end entity;


architecture func of TFTCtl is
	constant CLOCKFREQ : natural := 100; --MHZ
	constant TRESET : natural := 1; --ms
	constant TPOWERUP : natural := 2; --ms
	constant TLEDWARMUP : natural := 200; --ms
	constant TRESET_CYCLES : natural := natural(CLOCKFREQ*TRESET*1000);
	constant TPOWERUP_CYCLES : natural := natural(CLOCKFREQ*TPOWERUP*1000);
	constant TLEDWARMUP_CYCLES : natural := natural(CLOCKFREQ*TLEDWARMUP*1000);
	
	signal waitCnt : natural range 0 to TLEDWARMUP_CYCLES;
	signal waitCntEn : std_logic;

        signal clkStop : std_logic;
        signal clk10 : natural range 0 to 9;

	type state_type is (stOff, stPowerUp, stLEDWarmup, stOn); 
	signal state, nstate : state_type;

        signal int_R, int_G, int_B : std_logic_vector(7 downto 0);

        signal int_DE : std_logic;

        signal Xpixel : natural range 0 to 524;
        signal Ypixel : natural range 0 to 287;
        signal TFTRdy : std_logic;

begin


--  int_R <= (others => '1');
--  int_G <= (others => '1');  
--  int_B <= (others => '1');  

--  int_DE <= '1';

  int_R <= (others => '1') when (Xpixel>=0) and (XPixel<160) else (others => '0');
  int_G <= (others => '1') when (Xpixel>=160) and (XPixel<319) else (others => '0');
  int_B <= (others => '1') when (Xpixel>=320) and (XPixel<480) else (others => '0');  

  int_DE <= '0' when ((Xpixel >= 480) or (Ypixel >= 272)) else '1';
  

  --Wait States
  waitCntEn <= '1' when (state = stOff or state = stPowerUp or state = stLEDWarmup) and 
						  (state = nstate) else
					'0';

  --LCD & backlight power
  VDDEN_O <= '0' when (state = stOff) else '1';
  --Display On/Off signal
  DISP_O <= '0' when (state = stOff) or (state = StPowerUp) else '1';
  --Interface signals
  DE_O <= '0' when (state = stOff) or (state = stPowerUp) else int_DE;
  R_O <= (others => '0') when (state = stOff) or (state = stPowerUp) else int_R;
  G_O <= (others => '0') when (state = stOff) or (state = stPowerUp) else int_G;
  B_O <= (others => '0') when (state = stOff) or (state = stPowerUp) else int_B;   
   --Clock signal
  clkStop <= '1' when (state = stOff) or (state = stPowerUp) else '0';


----------------------------------------------------------------------------------
-- Startup state machine
----------------------------------------------------------------------------------	
   SYNC_PROC: process (clk)
   begin
      if rising_edge(clk) then
        if (rst='1') then
          state <= stOff;
        else
          state <= nstate;
        end if;
      end if;
   end process;	        			


   NEXT_STATE_DECODE: process (state, waitCnt,rst)
   begin
     nstate <= state;
     case (state) is
       when stOff => --wait for signals to stabilize
         if (waitCnt = TRESET_CYCLES) then
           nstate <= stPowerUp;
         end if;
       when stPowerUp => --turn power on first
         if (waitCnt = TPOWERUP_CYCLES) then
           nstate <= stLEDWarmup;
         end if;
       when stLEDWarmup => --turn on interface signals
         if (waitCnt = TLEDWARMUP_CYCLES) then
           nstate <= stOn;
         end if;
       when stOn => --stay in stOn, until reset
         if (rst = '1') then
           nstate <= stOff;
         end if;
       when others =>
         nstate <= stOff;
     end case;      
   end process;


----------------------------------------------------------------------------------
-- Wait Counter
----------------------------------------------------------------------------------	
  process(clk)
  begin
    if Rising_Edge(clk) then
      if (rst='1') or (waitCntEn = '0') then
        waitCnt <= 0;
      else
        waitCnt <= waitCnt + 1;
      end if;
    end if;
  end process;


----------------------------------------------------------------------------------
-- 10 MHz clock
----------------------------------------------------------------------------------	
  process(clk)
  begin
    if rising_edge(clk) then
      if (rst='1') or (clkStop='1') or (clk10 = 9) then
        clk10 <= 0;
      else
        clk10 <= clk10 + 1;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if (rst='1') or (clkStop='1') or (clk10 = 0) then
        CLK_O <= '0';
      elsif (clk10 = 5) then
        CLK_O <= '1';
      end if;
    end if;
  end process;

----------------------------------------------------------------------------------
-- TFT motor
----------------------------------------------------------------------------------	

  TFTRdy <= '0' when (state = stOff) or (state = stPowerUp) else '1';

  
  process(clk)
  begin
    if rising_edge(clk) then
      if (rst='1') or (TFTRdy='0') then
        Xpixel <= 0;
      else
        if (clk10 = 9) then
          if Xpixel = 524 then
            Xpixel <= 0;
          else
            Xpixel <= XPixel + 1;
          end if;
        end if;
      end if;
    end if;
  end process;



  process(clk)
  begin
    if rising_edge(clk) then
      if (rst='1') or (TFTRdy='0') then
        Ypixel <= 0;
      else
        if (clk10= 9) then
          if (Xpixel = 524) then
            if (YPixel = 287) then
              Ypixel <= 0;
            else
              Ypixel <= Ypixel + 1;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;
  

end architecture;


